/**
 * Converting .gtf files to .Bound, .GeneRange and .TSSPAS files.
 * Notice that all coordinates in .gtf file is 1-based, but in .Bound, .GeneRange and .Tsspas files, start coordinates are 0-based, and end coordinates are 1-based.
 */
#include <iostream>
#include <fstream>
#include <string>
#include <sstream>
#include <cstdio>
#include <cstdlib>
#include <climits>
#include <vector>

using namespace std;

const int DIRSTDOUT=0;

int writetoostream(ostream &ofsbound, ostream &ofsrange, ostream & ofstsspas, 
		string prevtranscript, string prevseqname, string prevstrand, 
		vector<int>& startvec, vector<int>& endvec){
			//gene boundary record
			if(startvec.size()==0 || endvec.size()==0){
				cout<<"Error: vector size: "<<startvec.size()<<" "<<endvec.size()<<endl;
				return -1;
			}
			for(int i=0;i<startvec.size();i++){
				ofsbound<<prevseqname<<"\t"<<prevstrand<<"\t"<<startvec[i]-1<<"\t0\n";
			}
			for(int i=0;i<endvec.size();i++){
				ofsbound<<prevseqname<<"\t"<<prevstrand<<"\t"<<endvec[i]<<"\t1\n";
			}
			//gene range files
			//remove double "s
			if(prevtranscript[0]=='"'){
				prevtranscript=prevtranscript.substr(1);
			}
			if(prevtranscript[prevtranscript.size()-1]==';'){
				prevtranscript=prevtranscript.substr(0,prevtranscript.size()-1);
			}
			if(prevtranscript[prevtranscript.size()-1]=='"'){
				prevtranscript=prevtranscript.substr(0,prevtranscript.size()-1);
			}
			ofsrange<<prevtranscript<<"\t"<<prevseqname<<"\t"<<prevstrand<<"\t"<<startvec[0]-1<<"\t"<<endvec[endvec.size()-1]<<"\t";
			for(int i=0;i<startvec.size();i++){
				ofsrange<<startvec[i]-1;
				if(i!=startvec.size()-1)
					ofsrange<<",";
			}
			ofsrange<<"\t";
			for(int i=0;i<endvec.size();i++){
				ofsrange<<endvec[i];
				if(i!=endvec.size()-1)
					ofsrange<<",";
			}
			ofsrange<<"\n";
			//TSS-PAS files
			ofstsspas<<prevtranscript<<"\t"<<startvec[0]-1<<"\t"<<endvec[endvec.size()-1]<<"\n";




	return 0;
}

int main(int argc, char* argv[]){
	
	cout<<"This program is used to convert gene annotation file (.gtf) into IsoInfer input files: .Bound, .GeneRange and .TSSPAS.\n";
	cout<<"These files are placed on the same folder as the .gtf file, with file extension .Bound, .GeneRange and .TSSPAS.\n";

	if(argc!=2){
		cerr<<"Error: Gene .gtf file needed.\n";
		cerr<<"Usage: convert2BGRTP [.gtf file]\n";
		return -1;
	}
	ifstream ifs(argv[1]);
	if(!ifs.is_open()){
		cerr<<"Error opening gtf files.\n";
		return -1;
	}
	string insfile(argv[1]);
	string outbound=insfile+".Bound";
	string outrange=insfile+".GeneRange";
	string outtsspas=insfile+".TSSPAS";
	ofstream ofsbound(outbound.c_str());
	ofstream ofsrange(outrange.c_str());
	ofstream ofstsspas(outtsspas.c_str());

	string oneline;
	vector<int> startvec,endvec;
	string prevtranscript="";
	string prevseqname="";
	string prevstrand="";
	while(!ifs.eof()){
		getline(ifs,oneline);
		if(oneline.size()<1)break;
		stringstream ss(oneline);
		string seqname,source,feature,start,end,score,strand,frame, geneid, geneidval, transid, transidval;
		ss>>seqname>>source>>feature>>start>>end>>score>>strand>>frame>>geneid>>geneidval>>transid>>transidval;

		
		if(feature=="exon"){
			int nstart=atoi(start.c_str());
			int nend=atoi(end.c_str());
			if(nend==0 || nend==INT_MAX || nend==INT_MIN){
				cerr<<"Warning: error converting start and end positions.\n";
			}
			//cout<<"1:"<<transidval<<",2:"<<prevtranscript<<endl;
				if(transidval!=prevtranscript){
					//write to files
					if(prevtranscript!=""){
						if(DIRSTDOUT==0)
							writetoostream(ofsbound,ofsrange,ofstsspas,
								prevtranscript, prevseqname, prevstrand,
								startvec,endvec);
						else
							writetoostream(cout,cout,cout,
								prevtranscript, prevseqname, prevstrand,
								startvec,endvec);
					}
	
					startvec.clear();
					endvec.clear();
					startvec.push_back(nstart);
					endvec.push_back(nend);
					prevtranscript=transidval;
					prevseqname=seqname;
					prevstrand=strand;
				}	
				else{
					startvec.push_back(nstart);
					endvec.push_back(nend);
				}
			
		}
		if(geneidval!=transidval){
			cout<<geneidval<<" "<<transidval<<endl;
		}
		//cout<<seqname<<" "<<source<<" "<<feature<<" "<<start<<" "<<end<<" "<<score<<" "<<strand<<" "<<frame<<" "<<geneid<<" "<<geneidval<<" "<<transid<<" "<<transidval<<endl;
	}
	//final writing
	if(prevtranscript!="")
		if(DIRSTDOUT==0)
			writetoostream(ofsbound,ofsrange,ofstsspas,
				prevtranscript, prevseqname, prevstrand,
				startvec,endvec);
		else
			writetoostream(cout,cout,cout,
				prevtranscript, prevseqname, prevstrand,
				startvec,endvec);
	
	
	ofsbound.close();
	ofsrange.close();
	ofstsspas.close();
	ifs.close();
	return 0;
}
